﻿using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.DotProlog.Execution;
using OpcLabs.DotProlog.Terms;

namespace OpcLabs.DotProlog.Test
{
    // Semi-deterministic predicates are predicates that either fail or succeed exactly once and, for well behaved predicates, 
    // leave no choicepoints. 
    public class SemiDeterministicPredicateTest : PredicateTest
    {
        #region Public Constructors

        public SemiDeterministicPredicateTest([NotNull] Functor functor) : base(functor)
        {
        }

        public SemiDeterministicPredicateTest([NotNull] string name, int arity) : base(name, arity)
        {
        }

        #endregion



        #region Static Private Methods

        [NotNull]
        static private string MakeMessage([CanBeNull] VariableDictionary variableDictionary)
        {
            return MakeMessage(variableDictionary != null);
        }

        [NotNull]
        static private string MakeMessage(bool success)
        {
            return System.String.Format(CultureInfo.CurrentCulture, "Expected success or failure: {0}.", success);
        }

        #endregion



        #region Public Methods

        public void Test(bool expected)
        {
            TestPredicate(SuccessOrFailure(expected), MakeMessage(expected));
        }

        public void Test([NotNull] IList<Term> arguments, [CanBeNull] VariableDictionary expected)
        {
            TestPredicate(arguments, SuccessOrFailure(expected), MakeMessage(expected));
        }

        public void Test([NotNull] Term arg, bool expected)
        {
            TestPredicate(arg, SuccessOrFailure(expected), MakeMessage(expected));
        }

        public void Test([NotNull] Term arg, [CanBeNull] VariableDictionary expected)
        {
            TestPredicate(arg, SuccessOrFailure(expected), MakeMessage(expected));
        }

        public void Test([NotNull] Term arg1, [NotNull] Term arg2, bool expected)
        {
            TestPredicate(arg1, arg2, SuccessOrFailure(expected), MakeMessage(expected));
        }

        public void Test([NotNull] Term arg1, [NotNull] Term arg2, [CanBeNull] VariableDictionary expected)
        {
            TestPredicate(arg1, arg2, SuccessOrFailure(expected), MakeMessage(expected));
        }

        public void TestError([NotNull] Term arg1, [NotNull] Term arg2, [NotNull] Term errorTerm)
        {
            try
            {
                Test(arg1, arg2, true);
            }
            catch (PrologException prologException)
            {
                Term ball = prologException.Ball;
                Trace.WriteLine(ball);
                Assert.IsNotNull(ball);
                var impDep = new Variable();
                Assert.IsNotNull(TermUtilities.UnifyWithoutOccursCheck(new NTerm("error") { errorTerm, impDep }, ball));
                return;
            }
            Assert.Fail();
        }

        #endregion
    }
}
