﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Linq;
using System.Runtime.Serialization;
using JetBrains.Annotations;
using OpcLabs.BaseLib.Collections.Generic;
using OpcLabs.BaseLib.Collections.Generic.Extensions;
using OpcLabs.DotProlog.Terms;

namespace OpcLabs.DotProlog.Test
{
    [Serializable]
    public class VariableDictionary 
        : Dictionary<Variable, Term>
    {
        #region Public Constructors

        public VariableDictionary() : base(ReferenceEqualityComparer<Variable>.Default)
        {
        }

        #endregion



        #region Protected Constructors

        /// <summary>Initializes a new instance of the class with serialized data.</summary>
        /// <param name="info">The SerializationInfo that holds the serialized object data.</param>
        /// <param name="context">The StreamingContext that contains contextual information about the source or destination. 
        /// </param>
        [SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods")]    // CC runtime checks in the body
        [SuppressMessage("Microsoft.Usage", "CA2214:DoNotCallOverridableMethodsInConstructors")]    // caused by Code Contracts
        protected VariableDictionary([NotNull] SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            // IMPROVE: this won't deserialize the proper EqualityComparer!
        }

        #endregion



        #region Public Methods

        [NotNull]
        [SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "s")]
        public Variable Add([NotNull] string s)
        {
            var variable = new Variable();
            Add(variable, Atom.Add(s));
            return variable;
        }

        [SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods")]    // CC runtime checks in the body
        public bool Match([NotNull] Solution solution)
        {
            if (!solution.SetEquals(/*this.*/Keys) && !solution.IsProperSupersetOf(Keys))
                return false;

            int i = this.FindIndex(pair =>
                // TODO: this should use variant/2 or =@=/2 instead
                // ReSharper disable PossibleNullReferenceException
                !pair.Key.Equals(pair.Value));
                // ReSharper restore PossibleNullReferenceException
            return i < 0;
        }

        public override string ToString()
        {
            return string.Join(
                "," + Environment.NewLine,
                this.Select(variableName =>
                    // ReSharper disable PossibleNullReferenceException
                    string.Format(CultureInfo.InvariantCulture,
                        "{0} = {1}",
                        variableName.Value,
                        (variableName.Key.Value == null) ? variableName.Key.ToString() : variableName.Key.Value.ToString()))
                    // ReSharper restore PossibleNullReferenceException
                );
        }

        #endregion



        #region Static Public Properties

        [NotNull]
        static public VariableDictionary Empty => new VariableDictionary();

        #endregion
    }
}
