﻿using System;
using System.Diagnostics;
using System.Reactive;
using System.ServiceModel;
using Microsoft.ComplexEventProcessing;
using Microsoft.ComplexEventProcessing.Linq;
using Microsoft.ComplexEventProcessing.ManagementService;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.DataAccess.ComplexEventProcessing;
using OpcLabs.EasyOpc.DataAccess.Reactive;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess.ComplexEventProcesing
{
    [TestClass]
    public class UnitTest1
    {
        // ReSharper disable LocalizableElement

        [TestCategory("ComplexEventProcessing")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void TestMethod1()
        {
            // Create an embedded StreamInsight server
            //using (var server = Server.Create("Default"))
            using (var server = CreateServer())
            {
                Debug.Assert(server != null);

                // Create a local end point for the server embedded in this program
                var managementService = server.CreateManagementService();
                Debug.Assert(managementService != null);
                var host = new ServiceHost(managementService);
                host.AddServiceEndpoint(typeof(IManagementService), new WSHttpBinding(SecurityMode.Message),
                    "http://localhost/MyStreamInsightServer");
                host.Open();

                /* The following entities will be defined and available in the server for other clients:
                 * serverApp
                 * serverSource
                 * serverSink
                 * serverProcess
                 */

                // CREATE a StreamInsight APPLICATION in the server
                var myApp = server.CreateApplication("serverApp");

                // DEFINE a simple SOURCE (returns a point event every second)
                const string machineName = "";
                const string serverClass = "OPCLabs.KitServer.2";
                const string itemId = "Simulation.Incrementing (1 s)";
                var observable = DAItemChangedObservable.Create<int>(machineName, serverClass, itemId, 100);
                var mySource = myApp
                    .DefineObservable(() => observable)
                    .ToPointStreamable(
                        eventArgs =>
                            PointEvent.CreateInsert(DateTimeOffset.Now,
                            (DAItemChangedPayload<int>)eventArgs),
                        AdvanceTimeSettings.StrictlyIncreasingStartTime);

                // DEPLOY the source to the server for clients to use
                mySource.Deploy("serverSource");

                // Compose a QUERY over the source (return every event carrying even data value)
                var myQuery = from e in mySource
                              where e.VtqPayload.Value % 2 == 0
                              select e;

                // DEFINE a simple observer SINK (writes the value to the server console)
                var mySink = myApp.DefineObserver(() => Observer.Create<DAItemChangedPayload<int>>(
                    payload => Console.WriteLine("sink_Server..: {0}", payload)));

                // DEPLOY the sink to the server for clients to use
                mySink.Deploy("serverSink");

                // BIND the query to the sink and RUN it
                var binding = myQuery.Bind(mySink);
                Debug.Assert(binding != null);
                using (/*var proc = */binding.Run("serverProcess"))
                {
                    // Wait...
                    Console.WriteLine("----------------------------------------------------------------");
                    Console.WriteLine("MyStreamInsightServer is running...");
                    Console.WriteLine("----------------------------------------------------------------");
                    Console.WriteLine(" ");
                    AssertUtilities.WaitingPhase(30 * 1000);
                    // IMPROVE: test output
                }
                host.Close();
            }
        }

        static private Server CreateServer()
        {
            try
            {
                return Server.Create("Instance1");
            }
            catch (InvalidOperationException exception)
            {
                if (exception.Message.StartsWith("The evaluation period for Microsoft StreamInsight instance 'Instance1' has expired."))
                    Assert.Inconclusive(exception.Message);
                if (exception.Message.StartsWith("The specified StreamInsight instance 'Instance1' could not be found or is not valid for this product."))
                    Assert.Inconclusive(exception.Message);
                throw;
            }
        }

        // ReSharper restore LocalizableElement
    }
}
