﻿using System;
using System.Reactive.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Licensing;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.Reactive;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess.Reactive
{
    public class DAReactiveTest
    {
        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes

        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //

        #endregion

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void ItemChangedObservableTest()
        {
            const string machineName = "";
            const string serverClass = "OPCLabs.KitServer.2";
            const string itemId = "Simulation.Incrementing (1 s)";

            var observable = DAItemChangedObservable.Create<int>(machineName, serverClass, itemId, 100);
            try
            {
                using (/*var subscription = */observable.Subscribe(e =>
                    // ReSharper disable PossibleNullReferenceException
                    AssertUtilities.WriteLine(e.TypedVtq.ToString())))
                    // ReSharper restore PossibleNullReferenceException
                    AssertUtilities.WaitingPhase(10 * 1000);
            }
            catch (LicensingException licensingException)
            {
                AssertUtilities.WriteLine(licensingException);
                Assert.Inconclusive();
            }
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void ItemChangedObservableTest_InvalidCast()
        {
            const string machineName = "";
            const string serverClass = "OPCLabs.KitServer.2";
            const string itemId = "Simulation.Random (1 s)";

            var observable = DAItemChangedObservable.Create<int>(
                new ServerDescriptor(machineName, serverClass), 
                new DAItemDescriptor(itemId), 
                100);
            try
            {
                using (/*var subscription = */observable.Subscribe(AssertUtilities.WriteLine))
                    AssertUtilities.WaitingPhase(10 * 1000);
            }
            catch (LicensingException licensingException)
            {
                AssertUtilities.WriteLine(licensingException);
                Assert.Inconclusive();
            }
        }

        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void PipelineTest()
        {
            const string machineName = "";
            const string serverClass = "OPCLabs.KitServer.2";
            const string sourceItemId = "Simulation.Incrementing (1 s)";
            const string destinationItemId = "Simulation.Register_I4";

            var sourceObservable = DAItemChangedObservable.Create<int>(machineName, serverClass, sourceItemId, 100);
            var processedObservable = sourceObservable.Select(e => 
                // ReSharper disable PossibleNullReferenceException
                e.TypedVtq.TypedValue*1000);
                // ReSharper restore PossibleNullReferenceException

            var observer = DAWriteItemValueObserver.Create<int>(machineName, serverClass, destinationItemId);
            var destinationObservable = DAItemChangedObservable.Create<int>(machineName, serverClass, destinationItemId, 100);

            // ReSharper disable once PossibleNullReferenceException
            using (/*var sourceSubscription = */processedObservable.Subscribe(observer))
                try
                {
                    using (/*var destinationSubscription = */destinationObservable.Subscribe(e =>
                        // ReSharper disable PossibleNullReferenceException
                        AssertUtilities.WriteLine(e.TypedVtq.ToString())))
                        // ReSharper restore PossibleNullReferenceException
                        AssertUtilities.WaitingPhase(10 * 1000);
                }
                catch (LicensingException licensingException)
                {
                    AssertUtilities.WriteLine(licensingException);
                    Assert.Inconclusive();
                }
        }
    }
}
