﻿using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.Navigation;
using OpcLabs.EasyOpc.UA.Navigation.Parsing;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcTest.UA.Navigation
{
    public partial class UABrowsePathTest
    {
        public class ParseProviderTest
        {
            #region Protected Properties

            protected UABrowsePathParsingFlags ParsingFlags { get; set; }

            #endregion



            #region Test Methods
            // ReSharper disable InconsistentNaming

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_Absolute_MultipleElements()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]/name0/name1/name2", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(3, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                Assert.AreEqual(3, browsePathElements.Count);

                UABrowsePathElement browsePathElement0 = browsePathElements[0];
                Assert.IsNotNull(browsePathElement0);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name0")),
                    browsePathElement0);

                UABrowsePathElement browsePathElement1 = browsePathElements[1];
                Assert.IsNotNull(browsePathElement1);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name1")),
                    browsePathElement1);

                UABrowsePathElement browsePathElement2 = browsePathElements[2];
                Assert.IsNotNull(browsePathElement2);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name2")),
                    browsePathElement2);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_Absolute_SingleElement()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]/name0", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name0")),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_Absolute_SingleElement_WithBasePath()
            {
                var basePath = new UABrowsePath(UAObjectIds.DataTypesFolder);
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    basePath, "[ObjectsFolder]/name0", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name0")),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_AnyComponent()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder].name0", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyComponent, new UAQualifiedName("namespace1", "name0")),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_ElementNamespaceIndex_Zero()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]/0:name0", "namespace1");

                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                var expected = new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName(0, "name0"));
                Assert.AreEqual(expected, browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_ElementNamespaceIndex_NonZero()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags,
                    UABrowsePath.Null, "[ObjectsFolder]/2:name0", "namespace1");

                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                var expected = new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName(2, "name0"));
                Assert.AreEqual(expected, browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_ForwardReference()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]<AlwaysGeneratesEvent>name0", UAUtilities.OpcUANamespaceUriString);
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(new UABrowsePathElement(
                        UABrowsePathElementType.ForwardReference,
                        new UAQualifiedName(UABrowsePathParser.StandardNamespaceUriString, "name0"),
                        UAReferenceTypeIds.AlwaysGeneratesEvent,
                        includeSubtypes: true),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_ForwardReference_NoSubtypes()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]<#AlwaysGeneratesEvent>name0", UAUtilities.OpcUANamespaceUriString);
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(new UABrowsePathElement(
                        UABrowsePathElementType.ForwardReference,
                        new UAQualifiedName(UAUtilities.OpcUANamespaceUriString, "name0"),
                        UAReferenceTypeIds.AlwaysGeneratesEvent,
                        includeSubtypes: false),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_InverseReference()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]<!AlwaysGeneratesEvent>name0", UAUtilities.OpcUANamespaceUriString);
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(new UABrowsePathElement(
                        UABrowsePathElementType.InverseReference,
                        new UAQualifiedName(UAUtilities.OpcUANamespaceUriString, "name0"),
                        UAReferenceTypeIds.AlwaysGeneratesEvent,
                        includeSubtypes: true),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_InverseReference_NoSubtypes()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]<#!AlwaysGeneratesEvent>name0", UAUtilities.OpcUANamespaceUriString);
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(new UABrowsePathElement(
                        UABrowsePathElementType.InverseReference,
                        new UAQualifiedName(UAUtilities.OpcUANamespaceUriString, "name0"),
                        UAReferenceTypeIds.AlwaysGeneratesEvent,
                        includeSubtypes: false),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_ReferenceNamespaceIndex_NonZero()
            {
                try
                {
                    UABrowsePath.Parse(ParsingFlags, 
                        UABrowsePath.Null, "[ObjectsFolder]<1:AlwaysGeneratesEvent>name0", UAUtilities.OpcUANamespaceUriString);
                }
                catch (UABrowsePathFormatException exception)
                {
                    Assert.AreEqual(UABrowsePathFormatErrorCode.UnknownReferenceNamespaceIndex, exception.FormatError.Code);
                    return;
                }
                Assert.Fail();
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_ReferenceNamespaceIndex_Zero()
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    UABrowsePath.Null, "[ObjectsFolder]<0:AlwaysGeneratesEvent>name0", UAUtilities.OpcUANamespaceUriString);
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(new UABrowsePathElement(
                        UABrowsePathElementType.ForwardReference,
                        new UAQualifiedName(UAUtilities.OpcUANamespaceUriString, "name0"),
                        UAReferenceTypeIds.AlwaysGeneratesEvent,
                        includeSubtypes: true),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_Relative_MultipleElements()
            {
                var basePath = new UABrowsePath(UAObjectIds.DataTypesFolder);
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    basePath, "/name0/name1/name2", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(3, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.DataTypesFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                Assert.AreEqual(3, browsePathElements.Count);

                UABrowsePathElement browsePathElement0 = browsePathElements[0];
                Assert.IsNotNull(browsePathElement0);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name0")),
                    browsePathElement0);

                UABrowsePathElement browsePathElement1 = browsePathElements[1];
                Assert.IsNotNull(browsePathElement1);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name1")),
                    browsePathElement1);

                UABrowsePathElement browsePathElement2 = browsePathElements[2];
                Assert.IsNotNull(browsePathElement2);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name2")),
                    browsePathElement2);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_Relative_SingleElement()
            {
                var basePath = new UABrowsePath(UAObjectIds.DataTypesFolder);
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags, 
                    basePath, "/name0", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.DataTypesFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(
                    new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name0")),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            public void ParseTest_Relative_SingleElement_NoBasePath()
            {
                try
                {
                    UABrowsePath.Parse(ParsingFlags, UABrowsePath.Null, "/name0", "namespace1");
                }
                catch (UABrowsePathFormatException exception)
                {
                    Assert.AreEqual(UABrowsePathFormatErrorCode.RelativeWithNoBase, exception.FormatError.Code);
                    return;
                }
                Assert.Fail();
            }

            // ReSharper restore InconsistentNaming
            #endregion
        }

        [TestClass]
        // ReSharper disable InconsistentNaming
        public class UABrowsePathTest_ExtendedSyntax : ParseProviderTest
        // ReSharper restore InconsistentNaming
        {
            public UABrowsePathTest_ExtendedSyntax()
            {
                ParsingFlags = UABrowsePathParsingFlags.ExtendedSyntax;
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            // ReSharper disable InconsistentNaming
            public void ParseTest_ExtendedReference()
            // ReSharper restore InconsistentNaming
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags,
                    UABrowsePath.Null, "[ObjectsFolder]<[nsu=namespace2;s=reference0]>name0", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                Assert.AreEqual(new UABrowsePathElement(
                        UABrowsePathElementType.ForwardReference, 
                        new UAQualifiedName("namespace1", "name0"),
                        new UANodeId("namespace2", "reference0"), 
                        includeSubtypes:true),
                    browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            // ReSharper disable InconsistentNaming
            public void ParseTest_ExtendedReference_DefaultNamespace()
            // ReSharper restore InconsistentNaming
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags,
                    UABrowsePath.Null, "[ObjectsFolder]<[s=reference0]>name0", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                var expected = new UABrowsePathElement(
                    UABrowsePathElementType.ForwardReference,
                    new UAQualifiedName("namespace1", "name0"),
                    new UANodeId(UAUtilities.OpcUANamespaceUriString, "reference0"),
                    includeSubtypes: true);
                Assert.AreEqual(expected, browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            // ReSharper disable InconsistentNaming
            public void ParseTest_ExtendedTarget_NamespaceIndex_Zero()
                // ReSharper restore InconsistentNaming
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags,
                    UABrowsePath.Null, "[ObjectsFolder]/[ns=0;s=name0]", "namespace1");

                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                var expected = new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName(0, "name0"));
                Assert.AreEqual(expected, browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            // ReSharper disable InconsistentNaming
            public void ParseTest_ExtendedTarget_NamespaceIndex_NonZero()
                // ReSharper restore InconsistentNaming
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags,
                    UABrowsePath.Null, "[ObjectsFolder]/[ns=2;s=name0]", "namespace1");

                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                var expected = new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName(2, "name0"));
                Assert.AreEqual(expected, browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            // ReSharper disable InconsistentNaming
            public void ParseTest_ExtendedTarget_NamespaceUri()
            // ReSharper restore InconsistentNaming
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags,
                    UABrowsePath.Null, "[ObjectsFolder]/[nsu=namespace2;s=name0]", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                var expected = new UABrowsePathElement(UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace2", "name0"));
                Assert.AreEqual(expected, browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            // ReSharper disable InconsistentNaming
            public void ParseTest_ExtendedTarget_DefaultNamespace()
            // ReSharper restore InconsistentNaming
            {
                UABrowsePath browsePath = UABrowsePath.Parse(ParsingFlags,
                    UABrowsePath.Null, "[ObjectsFolder]/[s=name0]", "namespace1");
                Assert.IsNotNull(browsePath);
                Assert.AreEqual(1, browsePath.Length);
                Assert.IsFalse(browsePath.IsEmpty);
                Assert.IsFalse(browsePath.IsNull);
                Assert.AreEqual(UAObjectIds.ObjectsFolder, browsePath.StartingNodeId);
                UABrowsePathElementCollection browsePathElements = browsePath.Elements;
                Assert.IsNotNull(browsePathElements);
                UABrowsePathElement browsePathElement = browsePathElements.Single();
                Assert.IsNotNull(browsePathElement);
                var expected = new UABrowsePathElement(
                    UABrowsePathElementType.AnyHierarchical, 
                    new UAQualifiedName(UABrowsePathParser.StandardNamespaceUriString, "name0"));
                Assert.AreEqual(expected, browsePathElement);
            }

            [TestCategory("OpcUA")]
            [TestMethod]
            // ReSharper disable InconsistentNaming
            public void ParseTest_DiggerFound()
            // ReSharper restore InconsistentNaming
            {
                try
                {
                    // ReSharper disable ReturnValueOfPureMethodIsNotUsed
                    UABrowsePath.Parse("[", "[");
                    // ReSharper restore ReturnValueOfPureMethodIsNotUsed
                }
                catch (UABrowsePathFormatException browsePathFormatException)
                {
                    string message = browsePathFormatException.Message;
                    AssertUtilities.WriteLine(message);
                    StringAssert.Contains(message, "character");
                    StringAssert.Contains(message, "ending");
                    StringAssert.Contains(message, "argument");
                    StringAssert.Contains(message, "expected");
                    return;
                }
                Assert.Fail();
            }
        }

        [TestClass]
        // ReSharper disable InconsistentNaming
        public class UABrowsePathTest_StandardSyntax : ParseProviderTest
        // ReSharper restore InconsistentNaming
        {
            public UABrowsePathTest_StandardSyntax()
            {
                ParsingFlags = UABrowsePathParsingFlags.None;
            }
        }



        #region Test Methods
        // ReSharper disable InconsistentNaming

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToStringTest_Empty()
        {
            var target = new UABrowsePath(UAObjectIds.ObjectsFolder);
            string s = target.ToString();
            Assert.AreEqual("[ObjectsFolder]", s);
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToStringTest_SingleElement()
        {
            var target = new UABrowsePath(UAObjectIds.ObjectsFolder);
            target.AppendElement(new UABrowsePathElement(
                UABrowsePathElementType.AnyHierarchical, new UAQualifiedName("namespace1", "name1")));
            string s = target.ToString();
            Assert.AreEqual("[ObjectsFolder]/name1", s);
        }

        // ReSharper restore InconsistentNaming
        #endregion
    }
}
